package feud.testgame;

import java.util.*;

/**
 * This is a singleton used by the client to store events that should be transfered to the server in the next transfer.
 * An event can be death, firing,...<br>
 * The queue can contain events for different clients running in one VM.
 */
public class ClientEventQueue {

    private static ClientEventQueue instance=null;

    private List events=new ArrayList();

    /**
     * Creates a new queue.
     */
    private ClientEventQueue() {}

    /**
     * Returns the one and only instance. The queue is a singleton.
     * @return ClientEventQueue the queue
     */
    public static synchronized ClientEventQueue getInstance() {
        if (instance==null) {
            instance=new ClientEventQueue();
        }
        return instance;
    }

    /**
     * Adds an event to the queue.
     * @param event Event the event to add
     */
    public synchronized void add(Event event) {
        events.add(event);
    }

    /**
     * Clears the queue.
     */
    public synchronized void clear() {
        events.clear();
    }

    /**
     * Gets all events sheduled by a client.
     * @param clientID int the ID of the client
     * @return EventDataContainer the events packed into a data container
     */
    public synchronized EventDataContainer getEvents(int clientID) {
        EventDataContainer edc=new EventDataContainer();
        for(Iterator itty=events.iterator(); itty.hasNext();) {
            Event event=(Event) itty.next();
            event.setSourceClientID(clientID);
            edc.add(event);
        }
        return edc;
    }
}
