package feud.testgame.gui;

import com.threed.jpct.*;
import com.threed.jpct.util.*;
import java.util.*;
import java.net.*;
import feud.clientserver.*;
import feud.testgame.util.*;
import feud.testgame.*;

/**
 * Sets up and manages the server selection window which is displayed after startup and when pressing ESC while playing.
 */
public class ServerSelection implements DataChangeListener, GUIListener {

    private List servers = null;
    private Texture backDrop = null;
    private ServerEntry current = null;

    private Window window = null;
    private TextField field = null;
    private TextField portField = null;
    private Button startServer = null;
    private Button connect = null;
    private Button minimize = null;
    private Button close = null;
    private Label stateLabel = null;
    private Label portLabel = null;
    private Label connectLabel = null;
    private Label nameLabel = null;
    private TextField name=null;
    private Table tableLabels = null;
    private Table serverTable = null;

    private String state = "State: Looking for servers!";
    private GameClient client = null;

    /**
     * Creates a new instance using a given server browser and client instance. Albeit this behaves
     * like a GUIComponent in some ways, it isn't any.
     * @param sb the server browser as an instance of ServerBrowser
     * @param client the client as an instance of GameClient
     * @throws Exception
     */
    public ServerSelection(ServerBrowser sb, GameClient client) throws Exception {
        sb.addListener(this);
        TextureManager tm = TextureManager.getInstance();
        SimpleStream ss = new SimpleStream("window.gif");
        backDrop = new Texture(ss.getStream());
        ss.close();

        window = new Window(backDrop, 50, 50);

        field = new TextField(204, 388, 272, 16);

        portField = new TextField(203, 340, 108, 16);
        portField.setText(SimpleServer.DEFAULT_PORT + "");

        name=new TextField(204,409,125,16);
        String names=System.getProperty("user.name");
        if (names==null) {
            names="Player";
        }
        name.setText(names);
        nameLabel=new Label(85, 409);
        nameLabel.setText("Player name:");

        startServer = new Button(338, 340, 138, 19);
        startServer.setLabel("Start server");

        connect = new Button(338, 409, 138, 19);
        connect.setLabel("connect");

        close = new Button(477, 3, 18, 11);
        close.setHideLabel(true);
        close.setLabel("close");

        minimize = new Button(454, 3, 18, 11);
        minimize.setHideLabel(true);
        minimize.setLabel("minimize");

        close.setListener(this);
        minimize.setListener(this);
        startServer.setListener(this);
        connect.setListener(this);

        stateLabel = new Label(24, 450);
        stateLabel.setText(state);

        portLabel = new Label(84, 340);
        portLabel.setText("Server port:");

        connectLabel = new Label(94, 388);
        connectLabel.setText("Connect to:");

        tableLabels = new Table("Labels", 1, 3, 24, 22, 492, 38);
        tableLabels.setColumnSize(0, 176);
        tableLabels.setColumnSize(1, 231);
        tableLabels.setColumnSize(2, 60);
        tableLabels.setRowSize(0, 16);
        tableLabels.setCell(0, 0, "Server name");
        tableLabels.setCell(0, 1, "IP/Port");
        tableLabels.setCell(0, 2, "Players");

        serverTable = new Table("Servers", 17, 3, 24, 40, 492, 325);
        serverTable.setColumnSize(0, 176);
        serverTable.setColumnSize(1, 231);
        serverTable.setColumnSize(2, 60);
        serverTable.setListener(this);

        window.add(serverTable);
        window.add(tableLabels);
        window.add(portLabel);
        window.add(connectLabel);
        window.add(stateLabel);
        window.add(field);
        window.add(portField);
        window.add(connect);
        window.add(startServer);
        window.add(close);
        window.add(minimize);
        window.add(name);
        window.add(nameLabel);

        this.client = client;
    }

    public void dataChanged(Object data) {
        if (data instanceof List) {
            servers = (List) data;
            Collections.sort(servers);
            serverTable.clear();

            if (servers != null && servers.size() > 0) {
                int cnt = 0;
                for (Iterator itty = servers.iterator(); itty.hasNext(); ) {
                    ServerEntry se = (ServerEntry) itty.next();
                    serverTable.setCell(cnt, 0, se.getName());
                    serverTable.setCell(cnt, 1, se.getAddress().getHostAddress() + ":" + se.getPort());
                    serverTable.setCell(cnt, 2, new Integer(se.getClientCount()));
                    cnt++;
                }
            }

        } else {
            throw new RuntimeException("What the hell...???");
        }
    }

    /**
     * Enables/Disables the whole ServerSelection.
     * @param visible visible...or not...
     */
    public void setVisible(boolean visible) {
        window.setVisible(visible);
        if (client.isConnected()) {
            stateLabel.setText("State: connected!");
        } else {
            stateLabel.setText("State: Looking for servers!");
        }
    }

    /**
     * Is the ServerSelection visible?
     * @return boolean is it?
     */
    public boolean isVisible() {
        return window.isVisible();
    }

    /**
     * Processes key and mouse events. What this actually does, is to redirect such events to
     * the server selection window, which is a real GUIComponent which can process these events.
     * @param mouse the MouseMapper
     * @param keyMapper the KeyMapper
     */
    public void evaluateInput(MouseMapper mouse, KeyMapper keyMapper) {
        window.evaluateInput(mouse, keyMapper);
    }

    /**
     * Draws the ServerSelection.
     * @param buffer the FrameBuffer to draw to.
     */
    public void draw(FrameBuffer buffer) {
        if (client.isConnected()) {
            connect.setLabel("disconnect");
        } else {
            connect.setLabel("connect");
        }

        if (client.runsServer()) {
            startServer.setLabel("Stop server");
        } else {
            startServer.setLabel("Start server");
        }
        window.draw(buffer);
    }

    public void elementChanged(String label, String data) {
        // Connect!
        if (label.equals("connect")) {
            data = field.getText();
            int pos = data.indexOf(":");
            int port = SimpleServer.DEFAULT_PORT;
            if (pos != -1) {
                try {
                    port = Integer.parseInt(data.substring(pos + 1));
                } catch (Exception e) {}
                data = data.substring(0, pos);
            }
            try {
                current = new ServerEntry("unknown", InetAddress.getByName(data), port, 0);
                setVisible(false);
                client.connect(current, name.getText());
                stateLabel.setText("State: connected!");
            } catch (Exception e) {
                setVisible(true);
                stateLabel.setText("State: Can't connect to that server!");
                e.printStackTrace();
                return;
            }
        }

        // Disconnect
        if (label.equals("disconnect")) {
            if (client.isConnected()) {
                try {
                    client.disconnect();
                    stateLabel.setText("State: Looking for servers!");
                } catch (Exception e) {
                    stateLabel.setText("State: Can't disconnect from that server!");
                    e.printStackTrace();
                    return;
                }
            }
        }

        // Starting server
        if (label.equals("Start server")) {
            if (!client.runsServer()) {
                int port = SimpleServer.DEFAULT_PORT;
                try {
                    port = Integer.parseInt(portField.getText());
                } catch (Exception e) {}

                try {
                    client.startServer(port);
                    stateLabel.setText("State: Local server started!");
                } catch (Exception e) {
                    stateLabel.setText("State: Can't start local server!");
                    e.printStackTrace();
                    return;
                }
            }
        }

        // Stoping Server
        if (label.equals("Stop server")) {
            if (client.runsServer()) {
                try {
                    client.shutDownServer();
                    stateLabel.setText("State: Local server stopped!");
                } catch (Exception e) {
                    stateLabel.setText("State: Can't stop local server!");
                    e.printStackTrace();
                    return;
                }
            }
        }

        // Server selected
        if (label.equals("Servers")) {
            int pos = new Integer(data).intValue();
            if (servers != null && servers.size() > pos) {
                ServerEntry se = (ServerEntry) servers.get(pos);
                field.setText(se.getAddress().getHostAddress() + ":" + se.getPort());
            }
        }

        if (label.equals("close")) {
            client.quit();
        }

        if (label.equals("minimize")) {
            setVisible(false);
        }
    }
}
