package feud.testgame.gui;

import com.threed.jpct.*;
import com.threed.jpct.util.*;
import feud.testgame.util.*;

/**
 * A window component. A window can contain other components as a kind of master container and it can be dragged around with the mouse.
 * As all components in this "toolkit", the window doesn't render itself in a generic way. Its visuals are defined by a texture.
 */
public class Window extends GUIComponent {

    private Texture backDrop=null;

    private int xDrag=-999;
    private int yDrag=-999;
    private int width=0;

    /**
     * Creates a new window.
     * @param backDrop the texture that defines how the window looks like.
     * @param x the x-position of the upper left corner.
     * @param y the y-position of the upper left corner.
     * @throws Exception
     */
    public Window(Texture backDrop, int x, int y) throws Exception {
        setX(x);
        setY(y);
        this.backDrop=backDrop;
        width=backDrop.getWidth();
    }

    public boolean evaluateInput(MouseMapper mouse, KeyMapper keyMapper) {
        if (visible) {
            boolean has = super.evaluateInput(mouse, keyMapper);
            if (!has) {
                boolean input = false;
                int xpos = getX();
                int ypos = getY();
                int y = mouse.getMouseY();
                int x = mouse.getMouseX();

                if (x >= xpos && x <= width + xpos && y >= ypos && y <= 15 + ypos || xDrag != -999) {
                    // title bar:
                    if (mouse.buttonDown(0)) {
                        if (xDrag == -999) {
                            xDrag = x;
                            yDrag = y;
                        }
                        xpos += x - xDrag;
                        ypos += y - yDrag;
                        setX(xpos);
                        setY(ypos);
                        xDrag = x;
                        yDrag = y;
                        input = true;
                    } else {
                        xDrag = -999;
                        yDrag = -999;
                    }
                } else {
                    xDrag = -999;
                    yDrag = -999;
                }
                if (!input) {
                    while (keyMapper.poll()!=KeyState.NONE) {
                        // consume all keyboard events, because nothing else happened!
                    }
                }
                return input;
            } else {
                return has;
            }
        }
        return false;
    }

    public void draw(FrameBuffer buffer) {
        if (visible) {
            buffer.blit(backDrop, 0, 0, getX(), getY(), backDrop.getWidth(), backDrop.getHeight(), FrameBuffer.TRANSPARENT_BLITTING);
            super.draw(buffer);
        }
    }
}
