package feud.testgame.util;

import java.io.*;
import java.util.*;

import com.threed.jpct.*;

/**
 * Helper class to serialize level mesh data and textures. Because the actual example loads them as a serialized stream, it
 * could be required to recreate the serialized versions when jPCT versions change. Just run this class once in that case.
 */
public class LevelSerializer {

    public static void main(String[] args) throws Exception {
        serialize();
    }

    private static void serialize() throws Exception {
        TextureManager texMan = TextureManager.getInstance();
        File dir = new File("rawdata/");
        String[] files = dir.list();
        for (int i = 0; i < files.length; i++) {
            String name = files[i];
            if (name.toLowerCase().endsWith(".jpg")) {
                texMan.addTexture(name, new Texture("rawdata/" + name));
            }
        }

        Object3D[] levelParts = Loader.load3DS("rawdata/ql1.3ds", 5f);
        Object3D level = new Object3D(0);

        for (int i = 0; i < levelParts.length; i++) {
            Object3D part = levelParts[i];
            part.setCenter(SimpleVector.ORIGIN);
            part.rotateX((float) - Math.PI / 2);
            part.rotateMesh();
            part.setRotationMatrix(new Matrix());

            level = Object3D.mergeObjects(level, part);
        }

        levelParts = Loader.load3DS("rawdata/ql2.3ds", 5f);

        Object3D lightmap = new Object3D(0);

        for (int i = 0; i < levelParts.length; i++) {
            Object3D part = levelParts[i];
            part.setCenter(SimpleVector.ORIGIN);
            part.rotateX((float) - Math.PI / 2);
            part.rotateMesh();
            part.setRotationMatrix(new Matrix());

            lightmap = Object3D.mergeObjects(lightmap, part);
        }

        PolygonManager pm1 = level.getPolygonManager();
        PolygonManager pm2 = lightmap.getPolygonManager();

        int max = pm1.getMaxPolygonID();

        Object3D result = new Object3D(max);

        for (int i = 0; i < max; i++) {
            int tid1 = pm1.getPolygonTexture(i);
            int tid2 = pm2.getPolygonTexture(i);
            SimpleVector ver1 = pm1.getTransformedVertex(i, 0);
            SimpleVector ver2 = pm1.getTransformedVertex(i, 1);
            SimpleVector ver3 = pm1.getTransformedVertex(i, 2);
            SimpleVector lt1 = pm1.getTextureUV(i, 0);
            SimpleVector lt2 = pm1.getTextureUV(i, 1);
            SimpleVector lt3 = pm1.getTextureUV(i, 2);

            SimpleVector mt1 = pm2.getTextureUV(i, 0);
            SimpleVector mt2 = pm2.getTextureUV(i, 1);
            SimpleVector mt3 = pm2.getTextureUV(i, 2);

            TextureInfo ti = new TextureInfo(tid1, lt1.x, lt1.y, lt2.x, lt2.y, lt3.x, lt3.y);
            ti.add(tid2, mt1.x, mt1.y, mt2.x, mt2.y, mt3.x, mt3.y, TextureInfo.MODE_MODULATE);
            result.addTriangle(ver1, ver2, ver3, ti);
        }

        level = result;

        OcTree ocb = new OcTree(level, 300, OcTree.MODE_OPTIMIZED);

        level.setOcTree(ocb);
        level.build();

        Vector textures = TextureManager.getInstance().getState();

        ObjectOutput out = new ObjectOutputStream(new FileOutputStream("data/level.ser"));
        out.writeObject(level);
        out.close();

        out = new ObjectOutputStream(new FileOutputStream("data/textures.ser"));
        out.writeObject(textures);
        out.close();
    }
}
