package feud.testgame.util;

/**
 * A simple ticker class for measuring in-game-time. This was formerly based on the LWJGL Timer class, but that class
 * has some unfixed problems on some machines/VMs...so it's based on Java5's nanotime now.
 */
public class Ticker {
    private int rate;
    private long s,s2;

    /**
     * Create a new ticker that ticks every n-th millisecond.
     * @param tickrateMS the interval
     */
    public Ticker(int tickrateMS) {
        rate = tickrateMS;
        s2 = System.nanoTime()/1000000L;
    }

    /**
     * Reset the ticker.
     */
    public void reset() {
        s2=System.nanoTime()/1000000L;
    }

    /**
     * Forward the ticker to the next tick.
     */
    public void forward() {
        s2=System.nanoTime()/1000000L-rate;
    }

    /**
     * How many ticks have passed since the last call?
     * @return int number of ticks
     */
    public int getTicks() {
        long i = System.nanoTime()/1000000L;
        if (i - s2 > rate) {
            int ticks = (int) ((i - s2) / (long) rate);
            s2 += rate * ticks;
            return ticks;
        }
        return 0;
    }
}
