package feud.testgame.view;

import com.threed.jpct.*;
import feud.testgame.model.*;


/**
 * An animated object is an extended client object which can use the animation information already present
 * in the client object to play a keyframe animation!
 */
public abstract class AnimatedObject extends ClientObject {

    public AnimatedObject() {
        super(Object3D.createDummyObj());
    }

    public AnimatedObject(Object3D obj) {
        super(obj);
    }

    public AnimatedObject(Object3D obj, Object3D child) {
        super(obj, child);
    }

    /**
     * Updates the view, i.e. animates the object and does the movement interpolation based on linear interpolation
     * until a new state from the server arrives.
     * @param ticks the ticks passed since the last call
     * @param level the level. Not used in within this method ATM.
     */
    public void process(long ticks, Level level) {
        int animSeq = getBackAnimation();
        int animSpeed = getBackAnimationSpeed();

        float offset = 0;
        boolean stop = false;
        if (animSeq > Animations.NOLOOP) {
            animSeq -= Animations.NOLOOP;
            stop = true;
        }

        if (isModified()) {
            SimpleVector pos = getBackPosition();
            getTranslationMatrix().setIdentity();
            translate(pos);
            setModified(false);
        } else {
            SimpleVector spd = new SimpleVector(getBackSpeed());
            spd.scalarMul(ticks);
            translate(spd);
        }

        Matrix tar = getRotationMatrix().cloneMatrix();
        tar.interpolate(getRotationMatrix(), getBackRotationMatrix(), 0.2f * (float) ticks);
        setRotationMatrix(tar);

        int mul = 1;
        if (!getBackSpeed().equals(SimpleVector.ORIGIN) && getBackSpeed().calcDot(getBackRotationMatrix().getZAxis()) < 0) {
            mul = -1;
        }

        if (getLastSequence() != animSeq) {
            resetTicks();
        }

        setLastSequence(animSeq);
        setAnimSpeed(animSpeed);

        if (animSeq == Animations.DIE) {
            offset = 0.631f; // The death animation that we want to use starts here (why not at 0? Only md2 knows...):
            setClampingMode(Animation.USE_CLAMPING);
        } else {
            setClampingMode(Animation.USE_WRAPPING);
        }

        long tickCnt = getTicks();
        if (animSpeed != 0) {
            if (!stop) {
                animate((float) (tickCnt % animSpeed) / (float) animSpeed + offset, animSeq);
            } else {
                animate(1f, animSeq);
            }
        }
        addToTicks(ticks * mul);
    }
}
