package feud.testgame.view;

import com.threed.jpct.*;

import feud.testgame.model.*;

/**
 * A collision listener for bullets. This listener is used to detect collisions of bullets with obstacles like walls and
 * similar. As it does is to flag these bullets as "collided" so that they can be removed later.
 */
public class BulletCollisionListener implements CollisionListener {

    private LocalBullet myBullet=null;

    /**
     * Create a new listener. Sadly, the listeners are working with view, not with model objects. This is why we
     * need to give it its model object and bound it to it. Not nice but not that bad either.
     * @param bullet the local bullet object that represents this bullet on the server and other clients.
     */
    public BulletCollisionListener(LocalBullet bullet) {
        myBullet=bullet;
    }

    public boolean requiresPolygonIDs() {
        return false;
    }

    public void collision(CollisionEvent ce) {
        if (ce.getType()==CollisionEvent.TYPE_SOURCE) {
            ClientObject co = (ClientObject) ce.getObject();
            if (co instanceof BulletView) {
                myBullet.setCollided(true);
            }
        }
    }
}
