package feud.testgame.view;

import com.threed.jpct.*;
import java.util.*;


/**
 * A small and dump factory that creates client objects. It can create one kind player model (includes the weapon) and one bullet model.
 * That's sufficient for this example "game" but not enough for a real one.
 */
public class ClientObjectFactory {

    /**
     * Type: player
     */
    public final static int PLAYER = 2;

    /**
     * Type: bullet
     */
    public final static int BULLET = 4;

    private final static int WEAPON = 3;
    private static ClientObjectFactory instance = null;
    private Map bluePrints = new HashMap();

    private ClientObjectFactory() {
        try {
            EnemyView e = new EnemyView();
            rotate(e);
            bluePrints.put(new Integer(PLAYER), e);

            WeaponView w = new WeaponView();
            rotate(w);
            bluePrints.put(new Integer(WEAPON), w);

            BulletView b = new BulletView();
            bluePrints.put(new Integer(BULLET), b);

        } catch (Exception e) {
            // Should not happen...
            throw new RuntimeException(e);
        }
    }

    /**
     * Get the instance. The Factory is a singleton.
     * @return ClientObjectFactory the one and only instance
     */
    public static synchronized ClientObjectFactory getInstance() {
        if (instance == null) {
            instance = new ClientObjectFactory();
        }
        return instance;
    }

    /**
     * Creates a new vlient object of the given type.
     * @param type the type
     * @return ClientObject the new object
     */
    public ClientObject create(int type) {
        try {
            ClientObject obj = null;
            switch (type) {
            case (PLAYER):
                obj = new EnemyView((Object3D) bluePrints.get(new Integer(type)), (Object3D) bluePrints.get(new Integer(WEAPON)));
                obj.setCollisionMode(Object3D.COLLISION_CHECK_OTHERS);
                obj.setCollisionOptimization(Object3D.COLLISION_DETECTION_OPTIMIZED);
                break;
            case (BULLET):
                obj = new BulletView((Object3D) bluePrints.get(new Integer(type)));
                break;
            default:
                throw new RuntimeException("Unknown type: " + type);
            }
            return obj;
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    private void rotate(Object3D obj) {
        obj.rotateY( -(float) Math.PI / 2f);
        obj.rotateMesh();
        obj.setRotationMatrix(new Matrix());
        obj.build();
    }
}
