package feud.testgame.view;

import com.threed.jpct.*;

/**
 * A simple manager class for managing decals. Decals are easy to handle because they exist on the client only.
 */
public class DecalManager {

   private int maxDecals=20;

   private Decal[] decals=null;

   /**
    * Create a new decal manager with a default number of decals.
    * @param world the world
    */
   public DecalManager(World world) {
      this(world, 20);
   }

   /**
    * Create a new decal manager with a maximum number of decals.
    * @param level the world.
    * @param maxDecals the max. number of decals
    */
   public DecalManager(World level, int maxDecals) {
      this.maxDecals=maxDecals;
      decals=new Decal[maxDecals];

      for (int i=0; i<maxDecals; i++) {
         decals[i]=new Decal();
         level.addObject(decals[i]);
      }
   }

   /**
    * Sets all decals to invisible.
    */
   public void reset() {
      for (int i=0; i<maxDecals; i++) {
         decals[i].setVisibility(Object3D.OBJ_INVISIBLE);
      }
   }

   /**
    * Creates a new decal at the given position with the given orientation.
    * @param pos the position
    * @param normal the orientation. The decal will be placed perpendicular to the normal
    */
   public void createDecal(SimpleVector pos, SimpleVector normal) {
      Decal decal=getFreeDecal();
      if (decal!=null) {
         decal.place(pos);
         decal.setRotationMatrix(normal.getRotationMatrix());
      }
   }

   /**
    * Gets a decal from the managers internal decal array
    * @return Decal the decal
    */
   private Decal getFreeDecal() {
      long min=Long.MAX_VALUE;
      Decal decal=null;
      for (int i=0; i<maxDecals; i++) {
         if (!decals[i].getVisibility()) {
            return decals[i];
         }

         if (decals[i].getDecalID()<min) {
            min=decals[i].getDecalID();
            decal=decals[i];
         }
      }
      return decal;
   }
}
