package feud.testgame.view;

import com.threed.jpct.*;

/**
 * An Object3D that represents the actual level. This example "game" only has one level. If you want more,
 * you'll have to implement a kind of level management yourself.
 */
public class Level extends Object3D {

    private ParticleManager parMan=null;

    /**
     * Creates a new level based on another object.
     * @param data the base level
     */
    public Level(Object3D data) {
        super(data);
        enableLazyTransformations();
    }

    /**
     * Returns a simple checksum to ensure (well, not really...) that all clients are using the same level.
     * This has to be improved for "real" games to prevent cheating. For this example, it's fine the way it is.
     * @return int the checksum
     */
    public int getChecksum() {
        return getMesh().getTriangleCount()*getMesh().getUniqueVertexCount()+getMesh().getVertexCount();
    }

    /**
     * Enables the particle emitter for this level.
     * @param world the world
     */
    public void enableParticleEmitter(World world) {
         parMan=new ParticleManager(world, 1000);
    }

    /**
     * Returns the particle emitter if enabled. If not, it returns null.
     * @return ParticleManager the emitter or null
     */
    public ParticleManager getParticleManager() {
        return parMan;
    }
}
