package feud.testgame.view;

import com.threed.jpct.*;

/**
 * A collision listener that listens if a something has hit the level structure. In this example, this is
 * limited to bullets and used for drawing decals.
 */
public class LevelEntityListener implements CollisionListener {

    private DecalManager decalMan = null;

    /**
     * Creates a new listener. The decal manager is needed for placing the decals in case of a collision event between
     * level and bullet.
     * @param decalMan the decal manager
     */
    public LevelEntityListener(DecalManager decalMan) {
        this.decalMan = decalMan;
    }

    public boolean requiresPolygonIDs() {
        return true;
    }

    /**
     * Triggered if something hits the level structure. For local bullets, a decal will be processed by the decal manager in that
     * case.
     * @param e CollisionEvent
     */
    public void collision(CollisionEvent e) {
        if (e.getType() == CollisionEvent.TYPE_TARGET && e.getSource() != null) {
            Object3D obj = e.getSource();

            if (obj instanceof BulletView) {
                BulletView bullet = (BulletView) obj;
                Object3D level = e.getObject();
                level.disableCollisionListeners();
                bullet.disableCollisionListeners();
                SimpleVector za = bullet.getZAxis();
                SimpleVector tc = bullet.getTransformedCenter();
                tc = tc.calcSub(za); // a small offset...
                float d = level.calcMinDistance(tc, za, 50);
                SimpleVector zas = new SimpleVector(za);
                zas.scalarMul( -0.15f);
                if (d != Object3D.COLLISION_NONE) {
                    za.scalarMul(d);
                    za.add(zas);
                    tc.add(za);
                    int[] ids = e.getPolygonIDs();
                    if (ids != null && ids.length > 0) {
                        int id = ids[0];
                        SimpleVector n = e.getObject().getPolygonManager().getTransformedNormal(id);
                        decalMan.createDecal(tc, n);
                    }
                }
                level.enableCollisionListeners();
                bullet.enableCollisionListeners();
            }
        }
    }
}
