package feud.testgame;

import feud.clientserver.*;
import com.threed.jpct.*;
import feud.testgame.model.*;

/**
 * An extended data container that offers support for transfering vectors, matrices and such. The container
 * may contains multiple data sets.
 */
public class ExtendedDataContainer extends DataContainer {

    /**
     * Create a new container.
     */
    public ExtendedDataContainer() {
        super();
    }

    /**
     * Build a new extended data container from the content of a generic data container.
     * @param dc DataContainer the generic data container
     */
    public ExtendedDataContainer(DataContainer dc) {
        //@fixme: This way, the array is extracted two times...which is quite dumb!
        super(dc.getRawData());
        setClientInfo(dc.getClientInfo());
        setMessageType(dc.getMessageType());
    }

    /**
     * Add a matrix to the container.
     * @param mat Matrix the matrix to add
     */
    public void add(Matrix mat) {
        float[] rotDump = mat.getDump();
        for (int i = 0; i < rotDump.length; i++) {
            add(rotDump[i]);
        }
    }

    /**
     * Returns the next matrix from the container (if there is one, otherwise an identity matrix will be returned).
     * @return Matrix the matrix
     */
    public Matrix getMatrix() {
        if (hasData()) {
            float[] dump = new float[16];
            for (int i = 0; i < 16; i++) {
                dump[i] = getNextFloat();
            }
            Matrix m = new Matrix();
            m.setDump(dump);
            return m;
        } else {
            return new Matrix();
        }
    }

    /**
     * Adds a new SimpleVector to the container.
     * @param sv SimpleVector the vector
     */
    public void add(SimpleVector sv) {
        add(sv.x);
        add(sv.y);
        add(sv.z);
    }

    /**
     * Adds a new LocalObject to the container.
     * @param lo LocalObject the object
     */
    public void add(LocalObject lo) {
        add(lo.getClientID());
        add(lo.getObjectID());
        add(lo.getType());
        add(lo.getAnimation());
        add(lo.getAnimationSpeed());
        add(lo.getRotation());
        add(lo.getPosition());
        add(lo.getSpeed());
    }

    /**
     * Fills an existing local object with the data from the container. The id will be ignored.
     * @param lo LocalObject the object to fill
     */
    public void fillLocalObject(LocalObject lo) {
        skip(); // ID nicht!
        lo.setObjectID(getNextInt());
        lo.setType(getNextInt());
        lo.setAnimation(getNextInt());
        lo.setAnimationSpeed(getNextInt());
        lo.setRotation(getMatrix());
        lo.setPosition(getSimpleVector());
        lo.setSpeed(getSimpleVector());
    }


    /**
     * Creates and returns a new LocalObject based on the based in the container.
     * @return LocalObject the object
     */
    public LocalObject getLocalObject() {
        LocalObject lo = new LocalObject(getNextInt());
        lo.setObjectID(getNextInt());
        lo.setType(getNextInt());
        lo.setAnimation(getNextInt());
        lo.setAnimationSpeed(getNextInt());
        lo.setRotation(getMatrix());
        lo.setPosition(getSimpleVector());
        lo.setSpeed(getSimpleVector());
        return lo;
    }

    /**
     * Returns the next SimpleVector from the container (is there is one, otherwise a null-vector will be returned).
     * @return SimpleVector the vector
     */
    public SimpleVector getSimpleVector() {
        if (hasData()) {
            return new SimpleVector(getNextFloat(), getNextFloat(), getNextFloat());
        } else {
            return new SimpleVector();
        }
    }
}
