package feud.testgame.model;

import com.threed.jpct.*;

/**
 * An extended local object that represents a bullet.
 */
public class LocalBullet extends LocalObject {

    public final static long LIFE_TIME=3000;

    private long start=System.currentTimeMillis();

    /**
     * Create a new bullet. A bullet is initially created on a client but for bullets, that doesn't matter as they are short lived
     * entities without "history". Therefore, there's no clientID in the constructor.
     */
    public LocalBullet() {
        super();
    }

    /**
     * Move the bullet.
     * @param ticks number of "ticks" in game time that have passed since the last movement.
     */
    public void move(long ticks) {
        SimpleVector p=getPosition();
        SimpleVector s=getSpeed();
        s.scalarMul(ticks);
        p.add(s);
        setPosition(p);
    }

    /**
     * The bullet is old, i.e. it has reached the end of it lifecycle.
     * @return boolean is it old?
     */
    public boolean isOld() {
        boolean old=System.currentTimeMillis()-start>LIFE_TIME;
        return old;
    }
}
