package feud.testgame.view;

import feud.testgame.*;
import com.threed.jpct.*;

import feud.testgame.model.*;

/**
 * A listener that gets called if a player's bullet has hit another, remote player's model.
 */
public class ClientObjectCollisionListener implements CollisionListener {

    private LocalBulletManager lobMan = null;
    private LocalObject obj = null;

    /**
     * Create a new Listener
     * @param lbm the local bullet manager that manages all bullets that "belong" to this client.
     * @param obj the local object of the remote player's model
     */
    public ClientObjectCollisionListener(LocalBulletManager lbm, LocalObject obj) {
        this.obj = obj;
        lobMan = lbm;
    }

    public boolean requiresPolygonIDs() {
        return false;
    }

    public void collision(CollisionEvent ce) {
        if (ce.getType()==CollisionEvent.TYPE_TARGET) {
            Object3D target = ce.getObject();
            if (ce.getSource() instanceof BulletView) {
                if (target instanceof ClientObject && !(target instanceof BulletView)) {
                    LocalObject bul = lobMan.getLocalObjectToView((ClientObject) ce.getSource());
                    if (bul!=null) {
                        Event event = new Event(Event.BULLET_HIT, bul.getObjectID(), obj.getObjectID(), obj.getClientID());
                        ClientEventQueue.getInstance().add(event);
                    }
                }
            }
        }
    }
}
